/*
 * Copyright (c) 2012-2014 The Ohio State University.  All rights reserved.
 *  
 * For detailed copyright and licensing information, please refer to the
 * LICENSE.TXT in the top level directory.
 */

#ifndef UPCR_UCR_INLINES_H
#define UPCR_UCR_INLINES_H

#include <mpi.h>
#include "upcr_ucr.h"
#include "gasnet/mvapich2x-conduit/gasnet_ucr_collectives.h"
#include <limits.h>
#include <float.h>
#include <string.h>
#define UPCR_UCR_ERROR GASNET_UCR_ERROR

GASNETT_INLINE(if_floating_type)
int if_floating_type(ucr_reduce_type_t type)
{
    if ((type==ucr_f)  ||       (type == ucr_d)  ||  (type == ucr_ld))
        return 1;
    else
        return 0;
}

GASNETT_INLINE(size_of_type)
int size_of_type(ucr_reduce_type_t type)
{
    switch(type){
        case ucr_sc:
            return sizeof(signed char);
            break;
        case ucr_uc:
            return sizeof(unsigned char);
            break;
        case ucr_ss:
            return sizeof(signed short);
            break;
        case ucr_us:
            return sizeof(unsigned short);
            break;
        case ucr_si:
            return sizeof(signed int);
            break;
        case ucr_ui:
            return sizeof(unsigned int);
            break;
        case ucr_sl:
            return sizeof(signed long);
            break;
        case ucr_ul:
            return sizeof(unsigned long);
            break;
        case ucr_f:
            return sizeof(float);
            break;
        case ucr_d:
            return sizeof(double);
            break;
        case ucr_ld:
            return sizeof(long double);
            break;
        default:
            return 0;
    }
}


GASNETT_INLINE(_signed)
int _signed(ucr_reduce_type_t type)
{
    if ((type == ucr_uc) || (type == ucr_us) || (type == ucr_ui) || (type == ucr_ul))
        return 0;
    else
        return 1;
}

GASNETT_INLINE(upc_dtype_to_ucr_dtype)
int upc_dtype_to_ucr_dtype(ucr_reduce_type_t type)
{
    int type_size = size_of_type(type)*8;
    switch(type_size){
        case 64:
            if (if_floating_type(type))
                return MPI_DOUBLE;
            else if (_signed(type))
                return MPI_LONG;
            else
                return MPI_UNSIGNED_LONG;
            break;
        case 32:
            if (if_floating_type(type))
                return MPI_FLOAT;
            else if (_signed(type))
                return MPI_INT;
            else 
                return MPI_UNSIGNED;
            break;
        case 16:
            if (if_floating_type(type))
                return UNSUPPORTED_OP;
            else if (_signed(type))
                return MPI_SHORT;
            else 
                return MPI_UNSIGNED_SHORT;
            break;
        case 8: 
            if (if_floating_type(type))
                return UNSUPPORTED_OP;
            else if (_signed(type))       
                return MPI_CHAR;
            else 
                return MPI_UNSIGNED_CHAR;
            break;
        default:
            return UNSUPPORTED_OP;
    }
}


GASNETT_INLINE(upc_op_to_ucr_op)
int upc_op_to_ucr_op(upc_op_t op)
{
    switch(op){
        case UPC_AND:
            return MPI_BAND;
            break;
        case UPC_OR:
            return MPI_BOR;
            break;
        case UPC_XOR:
            return MPI_BXOR;
        case UPC_MAX:
            return MPI_MAX;
            break;
        case UPC_MIN:
            return MPI_MIN;
            break;
        case UPC_ADD:
            return MPI_SUM;
            break;
        case UPC_MULT:
            return MPI_PROD;
            break;
        case UPC_LOGAND:
            return MPI_LAND;
            break;
        case UPC_LOGOR:
            return MPI_LOR;
            break;
        default:
            return UNSUPPORTED_OP;
    }
}

GASNETT_INLINE(set_one)
int set_one(void *my_partial, ucr_reduce_type_t type){
    switch(type){
        case ucr_sc:
            *(signed char *)my_partial = 1;
            break;
        case ucr_uc:
            *(unsigned char *)my_partial = 1;
            break;
        case ucr_ss:
            *(signed short *)my_partial = 1;
            break;
        case ucr_us:
            *(unsigned short *)my_partial = 1;
            break;
        case ucr_si:
            *(signed int *)my_partial = 1;
            break;
        case ucr_ui:
            *(unsigned int *)my_partial = 1;
            break;
        case ucr_sl:
            *(signed long *)my_partial = 1;
            break;
        case ucr_ul:
            *(unsigned long *)my_partial = 1;
            break;
        case ucr_f:
            *(float *)my_partial = 1;
            break;
        case ucr_d:
            *(double *)my_partial = 1;
            break;
        case ucr_ld:
            *(long double *)my_partial = 1;
            break;
        default:
            return 1;
    }
    return 0;
}

GASNETT_INLINE(set_max)
int set_max(void *my_partial, ucr_reduce_type_t type){
    switch(type){
        case ucr_sc:
            *(signed char *)my_partial = SCHAR_MAX;
            break;
        case ucr_uc:
            *(unsigned char *)my_partial = UCHAR_MAX;
            break;
        case ucr_ss:
            *(signed short *)my_partial = SHRT_MAX;
            break;
        case ucr_us:
            *(unsigned short *)my_partial = USHRT_MAX;
            break;
        case ucr_si:
            *(signed int *)my_partial = INT_MAX;
            break;
        case ucr_ui:
            *(unsigned int *)my_partial = UINT_MAX;
            break;
        case ucr_sl:
            *(signed long *)my_partial = LONG_MAX;
            break;
        case ucr_ul:
            *(unsigned long *)my_partial = ULONG_MAX;
            break;
        case ucr_f:
            *(float *)my_partial = FLT_MAX;
            break;
        case ucr_d:
            *(double *)my_partial = DBL_MAX;
            break;
        case ucr_ld:
            *(long double *)my_partial = LDBL_MAX;
            break;
        default:
            return 1;
    }
    return 0;
}

GASNETT_INLINE(set_min)
int set_min(void *my_partial, ucr_reduce_type_t type){
    switch(type){
        case ucr_sc:
            *(signed char *)my_partial = SCHAR_MIN;
            break;
        case ucr_uc:
            *(unsigned char *)my_partial = 0;
            break;
        case ucr_ss:
            *(signed short *)my_partial = SHRT_MIN;
            break;
        case ucr_us:
            *(unsigned short *)my_partial = 0;
            break;
        case ucr_si:
            *(signed int *)my_partial = INT_MIN;
            break;
        case ucr_ui:
            *(unsigned int *)my_partial = 0;
            break;
        case ucr_sl:
            *(signed long *)my_partial = LONG_MIN;
            break;
        case ucr_ul:
            *(unsigned long *)my_partial = 0;
            break;
        case ucr_f:
            *(float *)my_partial = FLT_MIN;
            break;
        case ucr_d:
            *(double *)my_partial = DBL_MIN;
            break;
        case ucr_ld:
            *(long double *)my_partial = LDBL_MIN;
            break;
        default:
            return 1;
    }
    return 0;
}

#endif
